@@ -1,8 +1,7 @@
 // Package buildinfo contains build information.
 //
-// Build information should be set during compilation by passing
-// -ldflags "-X src.elv.sh/pkg/buildinfo.Var=value" to "go build" or
-// "go get".
+// Some of the exported fields may be set during compilation by passing -ldflags
+// "-X src.elv.sh/pkg/buildinfo.Var=value" to "go build".
 package buildinfo
 
 import (
@@ -11,35 +10,42 @@ import (
 	"os"
 	"runtime"
 
+	"src.elv.sh/pkg/must"
 	"src.elv.sh/pkg/prog"
 )
 
-// Version identifies the version of Elvish. On development commits, it
+// VersionBase identifies the version of Elvish. On the development branches, it
 // identifies the next release.
-const Version = "0.18.0"
+const VersionBase = "0.19.0"
 
-// VersionSuffix is appended to Version to build the full version string. It is public so it can be
-// overridden when building Elvish; see PACKAGING.md for details.
-var VersionSuffix = ""
-
-// Reproducible identifies whether the build is reproducible. This can be
-// overridden when building Elvish; see PACKAGING.md for details.
-var Reproducible = "false"
+// BuildVariant may be set during compilation to identify a particular
+// build variant, such as a build by a specific distribution, with modified
+// dependencies, or with a non-standard toolchain.
+//
+// If non-empty, it is appended to the version string, along with a "+" prefix.
+var BuildVariant string
 
 // Type contains all the build information fields.
 type Type struct {
-	Version      string `json:"version"`
-	Reproducible bool   `json:"reproducible"`
-	GoVersion    string `json:"goversion"`
+	Version   string `json:"version"`
+	GoVersion string `json:"goversion"`
 }
 
 func (Type) IsStructMap() {}
 
 // Value contains all the build information.
 var Value = Type{
-	Version:      Version + VersionSuffix,
-	Reproducible: Reproducible == "true",
-	GoVersion:    runtime.Version(),
+	// On a release branch, change to addVariant(VersionBase, BuildVariant) and
+	// remove unneeded code.
+	Version:   addVariant(VersionBase, BuildVariant),
+	GoVersion: runtime.Version(),
+}
+
+func addVariant(version, variant string) string {
+	if variant != "" {
+		version += "+" + variant
+	}
+	return version
 }
 
 // Program is the buildinfo subprogram.
@@ -64,7 +70,6 @@ func (p *Program) Run(fds [3]*os.File, _ []string) error {
 		} else {
 			fmt.Fprintln(fds[1], "Version:", Value.Version)
 			fmt.Fprintln(fds[1], "Go version:", Value.GoVersion)
-			fmt.Fprintln(fds[1], "Reproducible build:", Value.Reproducible)
 		}
 	case p.version:
 		if *p.json {
@@ -73,15 +78,11 @@ func (p *Program) Run(fds [3]*os.File, _ []string) error {
 			fmt.Fprintln(fds[1], Value.Version)
 		}
 	default:
-		return prog.ErrNextProgram
+		return prog.NextProgram()
 	}
 	return nil
 }
 
-func mustToJSON(v interface{}) string {
-	b, err := json.Marshal(v)
-	if err != nil {
-		panic(err)
-	}
-	return string(b)
+func mustToJSON(v any) string {
+	return string(must.OK1(json.Marshal(v)))
 }
