@@ -8,26 +8,13 @@ import (
 	"src.elv.sh/pkg/parse"
 )
 
-//elvdoc:var -instant:binding
-//
-// Binding for the instant mode.
-
-//elvdoc:fn -instant:start
-//
-// Starts the instant mode. In instant mode, any text entered at the command
-// line is evaluated immediately, with the output displayed.
-//
-// **WARNING**: Beware of unintended consequences when using destructive
-// commands. For example, if you type `sudo rm -rf /tmp/*` in the instant mode,
-// Elvish will attempt to evaluate `sudo rm -rf /` when you typed that far.
-
 func initInstant(ed *Editor, ev *eval.Evaler, nb eval.NsBuilder) {
 	bindingVar := newBindingVar(emptyBindingsMap)
 	bindings := newMapBindings(ed, ev, bindingVar)
 	nb.AddNs("-instant",
 		eval.BuildNsNamed("edit:-instant").
 			AddVar("binding", bindingVar).
-			AddGoFns(map[string]interface{}{
+			AddGoFns(map[string]any{
 				"start": func() { instantStart(ed.app, ev, bindings) },
 			}))
 }
@@ -52,6 +39,6 @@ func instantStart(app cli.App, ev *eval.Evaler, bindings tk.Bindings) {
 		app.Redraw()
 	}
 	if err != nil {
-		app.Notify(err.Error())
+		app.Notify(modes.ErrorText(err))
 	}
 }
